/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _ATSTRINGORDERIVED_HPP
#define _ATSTRINGORDERIVED_HPP

#include <xercesc/util/XercesDefs.hpp>
#include <pathan/Pathan.hpp>
#include <pathan/AnyAtomicType.hpp>

class Collation;
class Sequence;
class Numeric;
class DynamicContext;

class PATHAN_EXPORT ATStringOrDerived : public AnyAtomicType
{

public:


  /* Get the name of the primitive type (basic type) of this type
   * (ie "decimal" for xs:decimal) */
  virtual const XMLCh* getPrimitiveTypeName() const = 0;

  /* Get the namespace URI for this type */
  virtual const XMLCh* getTypeURI() const = 0;

  /* Get the name of this type  (ie "integer" for xs:integer) */
  virtual const XMLCh* getTypeName() const = 0;

  /* returns the XMLCh* (canonical) representation of this type */
  virtual const XMLCh* asString(const DynamicContext* context) const = 0;

  /* returns this string as a set of codepoints */
  virtual const Sequence asCodepoints(const DynamicContext* context) const = 0;

  /* returns the length of this string */
  virtual unsigned int getLength() const = 0;

  /* returns the substring starting at startingLoc of given length */
  virtual const ATStringOrDerived* substring(const Numeric* startingLoc, const Numeric* length, const DynamicContext* context) const = 0;

  /* returns the substring that occurs after the first occurence of pattern */
  virtual const ATStringOrDerived* substringAfter(const ATStringOrDerived* pattern, Collation* collation, const DynamicContext* context) const = 0;

  /* returns the substring that occurs before the first occurence of pattern */
  virtual const ATStringOrDerived* substringBefore(const ATStringOrDerived* pattern, Collation* collation, const DynamicContext* context) const = 0;

  /* returns true if the two objects' URI are equal (string comparison)
   * false otherwise */
  virtual bool equals(const AnyAtomicType* target, const DynamicContext* context) const = 0;

  /** Releases the memory used by this Item */
  virtual void release() const = 0;

  virtual AnyAtomicType::AtomicObjectType getPrimitiveTypeIndex() const = 0;

protected: 
  /* Protected constructor */
  ATStringOrDerived(XPath2MemoryManager* memMgr): AnyAtomicType(memMgr) {};
};

#endif //  _ATSTRINGORDERIVED_HPP
